<?php

namespace Protobuf;

/**
 * Base interface implemented by Protocol MessageInterface objects.
 *
 * Fabio B. Silva <fabio.bat.silva@gmail.com>
 */
interface MessageInterface {

  /**
   * Creates message from the given stream.
   *
   * @param \Protobuf\Stream|resource|string $stream
   *   Stream.
   * @param \Protobuf\Configuration $configuration
   *   Configuration.
   *
   * @return \Protobuf\MessageInterface
   *   MessageInterface.
   */
  public static function fromStream($stream, Configuration $configuration = NULL);

  /**
   * Serializes the message and returns a stream containing its bytes.
   *
   * @param \Protobuf\Configuration $configuration
   *   Configuration.
   *
   * @return \Protobuf\Stream
   *   Stream.
   */
  public function toStream(Configuration $configuration = NULL);

  /**
   * Compute the number of bytes that would be needed to encode the message.
   *
   * @param \Protobuf\ComputeSizeContext $context
   *   Context.
   *
   * @return int
   *   Int.
   */
  public function serializedSize(ComputeSizeContext $context);

  /**
   * Serializes the message and returns a stream containing its bytes.
   *
   * @param \Protobuf\ReadContext $context
   *   Context.
   */
  public function readFrom(ReadContext $context);

  /**
   * Encodes and writes the message.
   *
   * @param \Protobuf\WriteContext $context
   *   Context.
   *
   * @return \Protobuf\Stream
   *   Stream.
   */
  public function writeTo(WriteContext $context);

  /**
   * Merge $context into the message being built.
   *
   * @param \Protobuf\MessageInterface $message
   *   MessageInterface.
   */
  public function merge(MessageInterface $message);

  /**
   * Obtain the list of unknown fields in this message.
   *
   * @return \Protobuf\UnknownFieldSet
   *   Unknown field set.
   */
  public function unknownFieldSet();

  /**
   * Obtain the map of extensions in this message.
   *
   * @return \Protobuf\Extension\ExtensionFieldMap
   *   ExtensionInterface field map.
   */
  public function extensions();

  /**
   * Reset all fields back to the initial values.
   */
  public function clear();

}
